// EmonLibrary examples openenergymonitor.org, Licence GNU GPL V3

/*
Mk1_in_garage_2 is based on Mk1_in_garage_day1 with modifications as follows:
- 1.5 second delay removed from end of loop()
- pin 10 setup to parallel pin 9 (PWM output)
- pin 13 LED to flash slowly when no surplus power is available
- safety margin reduced to 20W (the first Carlo Gavazzi step is approx 45W above this)
 
*/
#include "EmonLib.h"             // Include Emon Library
EnergyMonitor emon1;             // Create an instance

/* Although the PWM range is only 0 - 255, the control setting needs 
 * to be an integer to cope with out-of-range power values.
 */
int controlSetting = 0; 
int safetyMargin = 20;  // to minimise costs (Watts)
byte pin13LedState; // the on-board LED is toggled when no surplus power is available

const int controlOutput = 9; // PWM driver for CG controller
const int ledOutput = 10; // PWM driver for LED

void setup()
{  
  pinMode(controlOutput, OUTPUT);  // PWM driver for Carlo Gavazzi controller
  pinMode(ledOutput, OUTPUT);  // PWM driver for LED
  pinMode(2, OUTPUT); // for first quadrant's power indicator
  pinMode(3, OUTPUT); // for first quadrant's power indicator
  pinMode(4, OUTPUT); // for first quadrant's power indicator
  pinMode(5, OUTPUT); // for first quadrant's power indicator
  pinMode(13, OUTPUT); // on-board LED
  Serial.begin(9600);
  
  emon1.voltage(2, 295, 1.7);  // Voltage: input pin, calibration, phase_shift
  emon1.current(1, 78);       // Current: input pin, calibration.
  
}

void loop()
{
//  int measuredIncomingPower;
//  float powerForCalcs;
//  int stepsToChange;
//  byte ledFlag;
  
  emon1.calcVI(1000,2000);         // Calculate all. No.of wavelengths, time-out
  
//  Serial.print("Vrms ");
//  Serial.print(emon1.Vrms);
//  Serial.print("V;  ");
//  Serial.print(emon1.Irms);
//  Serial.print("A;  realPwr ");
//  Serial.print(emon1.realPower);
//  Serial.print("W;  ");
 
  int incomingPower = emon1.realPower; // rounded down to nearest Watt
  Serial.print("incomingPower = ");
  Serial.print(incomingPower);
  Serial.print("W;  ");

  /* Add a safety offset so that control system aims for a small level of export.
   * To avoid integer rounding a few lines on, powerForcalcs needs to be a 
   * floating point variable.
   */
  float powerForCalcs = incomingPower + safetyMargin; 
    
  /* Now calculate how much the control signal should be changed, up or down.
   * 3kW corresponds to 256 control steps.  Measured power is incoming, not exported, 
   * hence the minus sign.
   */
  int stepsToChange = -(powerForCalcs * 256 / 3000);
   
//  Serial.print(";  stepsToChange = ");
//  Serial.println(stepsToChange);
    
  controlSetting += stepsToChange;
    
  if (controlSetting < 0)
  {
    controlSetting = 0;
  }
  else
  if (controlSetting > 255)
  {
    controlSetting = 255;
  }
  Serial.print("controlSetting = ");
  Serial.print(controlSetting);
  Serial.print(", approx ");
  float controlSetting_float = controlSetting; // precision needed for maths
  int approxPower = controlSetting_float * 3000 / 256;
  Serial.print(approxPower);
  Serial.println("W to heater");
  analogWrite(controlOutput, controlSetting); // update the AC controller's driver
  analogWrite(ledOutput, controlSetting); // update the LED driver 

  /* turn on the indicator for the appropriate power quadrant 
   */
  byte ledFlag;
  PORTD &= B000011; // clear all four indicator pins (digital pins 2-5)
  digitalWrite(13, LOW); // make sure the on-board LED is off 
                         // to avoid it getting left on when power is available
  
  if (controlSetting == 0)   
    ledFlag = 0;
  else
    ledFlag = (controlSetting / 64) + 1;
  
  switch (ledFlag) 
  {
    case 0: // no power is available so toggle the onboard LED
      if(pin13LedState == HIGH) 
        pin13LedState = LOW;
      else 
        pin13LedState = HIGH;
        digitalWrite(13, pin13LedState);
      break;  
    case 1:
      PORTD |= B100; // illuminate 1st quadrant's power indicator (Dig pin 2) 
      break;
    case 2:
      PORTD |= B1000; // illuminate 2nd quadrant's power indicator (Dig pin 3) 
     break;
    case 3:
      PORTD |= B10000; // illuminate 3rd quadrant's power indicator (Dig pin 4)
      break;
    case 4:
      PORTD |= B100000;  // illuminate 4th quadrant's power indicator (Dig pin 5)
     break;
  } 
//  Serial.print("; pwr Ind -> ");
//  Serial.println(ledFlag);
  
//  delay (1500);
}
